@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
 /* logout.c - Ask user useful logout questions.
 
    Written by Owen Taylor <otaylor@redhat.com>
@@ -25,6 +26,8 @@
 #include <gtk/gtkinvisible.h>
 #include <gdk/gdkx.h>
 
+#include <gconf/gconf-client.h>
+
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 
@@ -32,28 +35,25 @@
 #include "logout.h"
 #include "command.h"
 #include "util.h"
+#include "gdm-logout-action.h"
 #include "gsm-multiscreen.h"
 #include "egg-screen-help.h"
 
-static gchar *halt_command[] =
+enum
 {
-  HALT_COMMAND, NULL
+  OPTION_LOGOUT,
+  OPTION_HALT,
+  OPTION_REBOOT
 };
 
-static gchar *reboot_command[] =
+static GConfEnumStringPair logout_options_lookup_table[] =
 {
-  REBOOT_COMMAND, NULL
+  { OPTION_LOGOUT, "logout"   },
+  { OPTION_HALT,   "shutdown" },
+  { OPTION_REBOOT, "restart"  },
+  { 0, NULL }
 };
 
-/* What action to take upon shutdown */
-static enum
-{
-  LOGOUT,
-  HALT,
-  REBOOT
-}
-action = LOGOUT;
-
 typedef struct {
   GdkScreen    *screen;
   int           monitor;
@@ -145,8 +145,7 @@ fadeout_callback (FadeoutData *fadeout)
 		       fadeout->gc,
 		       fadeout->end_pb,
 		       0, 0,
-		       fadeout->area.x,
-		       fadeout->area.y,
+		       0, 0,
 		       fadeout->area.width,
 		       fadeout->area.height,
 		       GDK_RGB_DITHER_NONE,
@@ -169,8 +168,7 @@ fadeout_callback (FadeoutData *fadeout)
 		   fadeout->gc,
 		   fadeout->frame,
 		   0, 0,
-		   fadeout->area.x,
-		   fadeout->area.y,
+		   0, 0,
 		   fadeout->area.width,
 		   fadeout->area.height,
 		   GDK_RGB_DITHER_NONE,
@@ -212,7 +210,7 @@ fadeout_screen (GdkScreen *screen,
   attr.override_redirect = TRUE;
   attr_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP | GDK_WA_NOREDIR;
 
-  fadeout->draw_window = gdk_window_new (NULL, &attr, attr_mask);
+  fadeout->draw_window = gdk_window_new (fadeout->root_window, &attr, attr_mask);
   fadeout_windows = g_list_prepend (fadeout_windows, fadeout->draw_window);
   
   fadeout->start_pb = gdk_pixbuf_get_from_drawable (NULL,
@@ -244,8 +242,7 @@ fadeout_screen (GdkScreen *screen,
 		   fadeout->gc,
 		   fadeout->frame,
 		   0, 0,
-		   fadeout->area.x,
-		   fadeout->area.y,
+		   0, 0,
 		   fadeout->area.width,
 		   fadeout->area.height,
 		   GDK_RGB_DITHER_NONE,
@@ -286,6 +283,40 @@ make_title_label (const char *text)
   return label;
 }
 
+static int
+get_default_option (void)
+{
+  GConfClient *gconf_client;
+  char        *str;
+  int          option;
+
+  gconf_client = gconf_client_get_default ();
+  str = gconf_client_get_string (gconf_client, LOGOUT_OPTION_KEY, NULL);
+
+  if (str == NULL || !gconf_string_to_enum (logout_options_lookup_table, str, &option))
+    option = OPTION_LOGOUT;
+  
+  g_object_unref (gconf_client);
+  g_free (str);
+  return option;
+}
+
+static void
+set_default_option (int option)
+{
+  GConfClient *gconf_client;
+  const char  *str;
+
+  gconf_client = gconf_client_get_default ();
+
+  str = gconf_enum_to_string (logout_options_lookup_table, option);
+  g_assert (str != NULL);
+
+  gconf_client_set_string (gconf_client, LOGOUT_OPTION_KEY, str, NULL);
+
+  g_object_unref (gconf_client);
+}
+
 static gboolean
 display_gui (void)
 {
@@ -296,18 +327,21 @@ display_gui (void)
   GtkWidget *image;
   GtkWidget *toggle_button = NULL;
   gint response;
-  gchar *s, *t;
   GtkWidget *halt = NULL;
   GtkWidget *reboot = NULL;
   GtkWidget *invisible;
-  gboolean   retval = FALSE;
+  gboolean halt_supported = FALSE;
+  gboolean reboot_supported = FALSE;
+  gboolean retval = FALSE;
   gboolean save_active = FALSE;
   gboolean halt_active = FALSE;
   gboolean reboot_active = FALSE;
+  GdmLogoutAction logout_action = GDM_LOGOUT_ACTION_NONE;
   gboolean a11y_enabled;
   GError *error = NULL;
   GdkScreen *screen;
   int monitor;
+  int selected_option;
 
   gsm_verbose ("display_gui: showing logout dialog\n");
 
@@ -395,19 +429,17 @@ display_gui (void)
 			  FALSE, TRUE, 0);
     }
 
-  /* Red Hat specific code to check if the user has a
-   * good chance of being able to shutdown the system,
-   * and if so, give them that option
-   */
-  s = g_strconcat ("/var/lock/console/", g_get_user_name (), NULL);
-  t = g_strconcat ("/var/run/console/", g_get_user_name (), NULL);
-  if (((geteuid () == 0) || g_file_exists (t) || g_file_exists(s)) &&
-      access (halt_command[0], X_OK) == 0)
+  halt_supported   = gdm_supports_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN);
+  reboot_supported = gdm_supports_logout_action (GDM_LOGOUT_ACTION_REBOOT);
+
+  if (halt_supported || reboot_supported)
     {
       GtkWidget *title, *spacer;
       GtkWidget *action_vbox, *hbox;
       GtkWidget *category_vbox;
       GtkWidget *r;
+
+      selected_option = get_default_option ();
       
       category_vbox = gtk_vbox_new (FALSE, 6);
       gtk_box_pack_start (GTK_BOX (vbox), category_vbox, TRUE, TRUE, 0);
@@ -431,19 +463,26 @@ display_gui (void)
       gtk_widget_show (action_vbox);
       
       r = gtk_radio_button_new_with_mnemonic (NULL, _("_Log out"));
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_LOGOUT));
       gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
       gtk_widget_show (r);
 
-      r = halt = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("Sh_ut down"));
-      gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
-      gtk_widget_show (r);
+      if (halt_supported)
+	{
+	  r = halt = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("Sh_ut down"));
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_HALT));
+	  gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
+	  gtk_widget_show (r);
+	}
 
-      r = reboot = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("_Restart the computer"));
-      gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
-      gtk_widget_show (r);
+      if (reboot_supported)
+	{
+	  r = reboot = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("_Restart the computer"));
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_REBOOT));
+	  gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
+	  gtk_widget_show (r);
+	}
     }
-  g_free (s);
-  g_free (t);
 
   gsm_center_window_on_screen (GTK_WINDOW (box), screen, monitor);
 
@@ -481,6 +520,13 @@ display_gui (void)
   if (toggle_button)
     save_active = GTK_TOGGLE_BUTTON (toggle_button)->active;
 
+  if (reboot_active)
+    selected_option = OPTION_REBOOT;
+  else if (halt_active)
+    selected_option = OPTION_HALT;
+  else
+    selected_option = OPTION_LOGOUT;
+
   gtk_widget_destroy (box);
   gtk_widget_destroy (invisible);
 
@@ -502,9 +548,10 @@ display_gui (void)
       if(save_active)
 	save_selected = save_active;
       if (halt_active)
-	action = HALT;
+	logout_action = GDM_LOGOUT_ACTION_SHUTDOWN;
       else if (reboot_active)
-	action = REBOOT;
+	logout_action = GDM_LOGOUT_ACTION_REBOOT;
+      set_default_option (selected_option);
       retval = TRUE;
       break;
     default:
@@ -544,6 +591,8 @@ display_gui (void)
       break;
     }
 
+  gdm_set_logout_action (logout_action);
+
   return retval;
 }
 
@@ -565,10 +614,8 @@ maybe_display_gui (void)
    */
   ice_frozen();
   result = display_gui ();
-  if (action == HALT)
-    set_logout_command (halt_command);
-  else if (action == REBOOT)
-    set_logout_command (reboot_command);
-  ice_thawed();
+
+  ice_thawed ();
+
   return result;
 }
